#!/usr/bin/env python3
#
# (c) Diladele B.V. 2018, Web Safety for Squid Proxy
#  
# ICAP transaction unpack script for ICAP debugging
#
import os
import sys
import logging
import argparse

class BinaryReader:
    def read_file(self, file):
        with open(file, mode='rb') as fin:
            return fin.read()
            
class BinaryWriter:
    def write_file(self, file, data):
        with open(file, mode='wb') as fout:
            fout.write(bytearray(data))
            
class HeaderParser:
    
    def parse(self, data_b):

        # debug check
        assert(type(data_b) == bytes)

        # the data is bytes, for header it is safe to convert it to string
        data = data_b.decode("utf-8")

        # ok let's go
        first   = None
        fields  = {}
            
        # parse HTTP like header
        lines = data.split("\r\n")
        for line in lines:
            
            line = line.strip().strip('\r\n')            
            if first is None:            
                first = line
            else:
                if len(line) == 0:
                    continue
                    
                pos = line.find(':')
                if pos == -1:
                    logging.warn("Ignoring line in HTTP like header as it does not contain ':' character - %s" % line)
                    
                name  = line[:pos]
                value = line[pos+1:]
                
                fields[name.strip()] = value.strip()
                
        # debug check
        assert(first is not None)
        assert(len(first) > 0)

        # and return
        return (first, fields)
                
class HttpRequestHeader:
    def __init__(self, data):    
        (self.first, self.fields) = HeaderParser().parse(data)
            
            
class HttpResponseHeader:
    def __init__(self, data):
        (self.first, self.fields) = HeaderParser().parse(data)
        
class DataReader:
    def __init__(self, data):

        assert(type(data) == bytes)

        self.data   = data
        self.offset = 0
        self.limit  = len(data)
        
    def pop(self):

        if self.offset >= self.limit:
            raise Exception
        else:
            char = self.data[self.offset]
            self.offset += 1            

            # it is an int
            return char

    def more_data_available(self):
        if self.offset < self.limit:
            return True
        return False

STATE_READING_CHUNK_SIZE        = 1
STATE_READING_CHUNK_END         = 2
STATE_READING_CHUNK_BODY_END1   = 3
STATE_READING_CHUNK_BODY_END2   = 4
STATE_READING_PREVIEW_END1      = 5
STATE_READING_PREVIEW_END2      = 6
STATE_READING_BODY_END1         = 7
STATE_READING_BODY_END2         = 8

class HttpResponseBody:

    def __init__(self, header, data):

        # debug check
        assert(type(data) == bytes)
        
        # the ICAP message contains HTTP response or HTTP request body encapsulated as chunked 
        # encoding with possible ICAP Preview, we do our parsing according to that
        
        preview_size = None
        if 'Preview' in header.fields:
            preview_size = int(header.fields['Preview'])
        
        # initial state and reader
        state  = STATE_READING_CHUNK_SIZE
        reader = DataReader(data)
        
        chunk_size   = bytearray()
        total_body   = []
        preview_done = False
            
        # do our parsing
        while True:
        
            c = reader.pop()
        
            if state == STATE_READING_CHUNK_SIZE:     

                if c == 0x0D: # \r'
                    state = STATE_READING_CHUNK_END
                else:
                    chunk_size.append(c)


            elif state == STATE_READING_CHUNK_END:

                if c == 0x0A: # \n

                    # debug check chunk is correct
                    assert(len(chunk_size) > 0)

                    # convert chunk to string to make it easy to work with
                    chunk_size_str = chunk_size.decode("utf-8")

                    # see if this is IEOF chank
                    size = None
                    ieof = False
                    if chunk_size_str == "0; ieof":
                        size = 0
                        ieof = True
                    else:
                        size = int(chunk_size_str, 16) # convert it into hex

                    # see what chunk we have
                    if size == 0:

                        # ok size of next chunk is zero, see if this is a preview
                        if preview_size is not None:

                            if not preview_done:

                                preview_done = True

                                if not ieof:

                                    # this is not ieof, just a simple preview
                                    assert(len(total_body) == preview_size)

                                    # it is finished, we have to consume \r\n
                                    state = STATE_READING_PREVIEW_END1
                                else:
                                    # this is ieof, the preview fully contains object
                                    state = STATE_READING_BODY_END1
                            else:
                                state = STATE_READING_BODY_END1
                        else:
                            raise Exception("E_NOTIMPL")

                            # no preview was indicated in the ICAP header, this means we have completed object parsing
                            state = STATE_READING_PARSING_COMPLETED
                    else:
                        # we got a nice chunk out, append to accumulated body
                        for i in range(0, size):
                            total_body.append(reader.pop())

                        # switch state
                        state = STATE_READING_CHUNK_BODY_END1

            elif state == STATE_READING_CHUNK_BODY_END1:

                if c == 0x0D: # \r
                    state = STATE_READING_CHUNK_BODY_END2
                else:
                    raise Exception("Invalid character '%s' when waiting for chunk body end" % char)

            elif state == STATE_READING_CHUNK_BODY_END2:

                if c == 0x0A: # \n

                    chunk_size = bytearray()
                    state      = STATE_READING_CHUNK_SIZE

                else:
                    raise Exception("Invalid character '%s' when waiting for chunk body end" % char)

            elif state == STATE_READING_PREVIEW_END1:

                if c == 0x0D: # \r
                    state = STATE_READING_PREVIEW_END2
                else:
                    raise Exception("Invalid character '%s' when waiting for preview end1" % char)

            elif state == STATE_READING_PREVIEW_END2:

                if c == 0x0A: # \n

                    chunk_size = bytearray()

                    # preview has ended, it may be that after scanning the preview we decided we are not
                    # interested in this object and sent Not Interesting reply to Squid. In this case
                    # Squid will not send us anything and data will now be empty.
                    if reader.more_data_available():
                        state = STATE_READING_CHUNK_SIZE
                    else:
                        # to be on a safe side
                        assert(len(total_body) == preview_size)

                        # break from the loop as nothing is left to be parsed. Please note the total_body contains only preview bytes
                        break
                else:
                    raise Exception("Invalid character '%s' when waiting for preview end2" % char)

            elif state == STATE_READING_BODY_END1:

                if c == 0x0D: # \r
                    state = STATE_READING_BODY_END2
                else:
                    raise Exception("Invalid character '%s' when waiting for body end1" % char)

            elif state == STATE_READING_BODY_END2:

                if c == 0x0A: # \n

                    # debug check all data is parsed
                    all_parsed = False
                    try:
                        reader.pop()
                    except:
                        all_parsed = True

                    assert(all_parsed)

                    # and break from the loop
                    break

                else:
                    raise Exception("Invalid character '%s' when waiting for body end2" % char)

            else:
                raise Exception("Invalid state '%s'" % str(state))


        # if we got here then the body is fully parsed, assign it
        self.body = total_body
            
class Encapsulated:

    def __init__(self, data):
    
        # order is important
        self.parts = []
        
        array = data.split(',')
        for item in array:
            
            # get the type and offset
            (type, offset) = item.strip().split('=')
            
            # and put in into list conserving the order
            self.parts.append( (type.strip(), int(offset)) )
            
            
class IcapHeader:

    def __init__(self, data):
    
        # see what ICAP message we have - request or response
        is_req = False
        if data.startswith("REQMOD"):
            is_req = True
        if not data.startswith("RESPMOD"):
            raise Exception("ICAP header does not start with either REQMOD or RESPMOD (it starts with %s)" % data[:6])

        first_line  = True
        self.fields = {}
            
        # parse the icap header
        lines = data.split("\r\n")
        for line in lines:
            
            line = line.strip().strip("\n")
            if first_line:
                first_line = False
            else:
                # empty line is not interesting
                if not line:
                    continue

                # ok line is not empty
                pos = line.find(":")
                if pos == -1:
                    logging.warn("Ignoring line in ICAP header as it does not contain ':' character - %s" % line)
                    continue

                name  = line[:pos]
                value = line[pos+1:]
                
                self.fields[name.strip()] = value.strip()

        # now we have the fields, parse the encapsulated info
        self.encapsulated = Encapsulated(self.fields["Encapsulated"])
        
class IcapBody:
    
    def __init__(self, header, data):
    
        # parse the ICAP body parts
        for i in range(0, len(header.encapsulated.parts)):
        
            # parse the type data tuple
            (type, content) = (None, None)
        
            part = header.encapsulated.parts[i]
            if i + 1 < len(header.encapsulated.parts):
                next  = header.encapsulated.parts[i+1]

                (type1, value1) = part
                (type2, value2) = next
                
                assert(value2 > value1)
                
                (type, content) = (type1, data[value1:value2])
            else:
                (type1, value1) = part                            
                (type, content) = (type1, data[value1:])
                
            # analyze the parsed tuple            
            if type == 'req-hdr':
                self.req_hdr = HttpRequestHeader(content)
            elif type == 'res-hdr':
                self.res_hdr = HttpResponseHeader(content)
            elif type == 'res-body':
                self.res_body = HttpResponseBody(header, content)
            else:
                raise Exception("Cannot parse ICAP body, unknown ICAP encapsulated entity type %s" % type)
            
class Message:
    pass
    
class RespMod(Message):
    pass
    
class ReqMod(Message):
    pass
    
class Parser:
    
    def parse(self, file):
    
        # read file as bytes
        data  = BinaryReader().read_file(file)        
        
        # these are the infixes we are searching for
        infix     = b"\r\n\r\n"
        infix_len = len(infix)
        
        # get the ICAP header
        pos = data.find(infix)
        if pos == -1:
            raise Exception("Cannot find the ICAP header in %s" % data[:100])

        # get the encapsulated message part
        header = data[:pos+infix_len]
        body   = data[pos+infix_len:]

        # we know the header is textual, so convert it
        header = header.decode("utf-8")

        # and leave body as bytes for now        
        return self.parse_icap(header, body)
        
    def parse_icap(self, header, body):

        # debug check
        assert(type(header) == str)
        assert(type(body)   == bytes)

        icap_header = IcapHeader(header)        
        icap_body   = IcapBody(icap_header, body)

        return (icap_header, icap_body)
        
class RespModParser(Parser):
    pass
        
def unpack(file):

    # construct unpacked file names
    base_name = os.path.splitext(file)[0]

    # parse the icap header and body
    (icap_header, icap_body) = RespModParser().parse(file)
    
    # write the body
    if hasattr(icap_body, 'res_body'):

        # ending
        ending = ".gz"

        # currently the body might be gzip or br
        encoding  = icap_body.res_hdr.fields.get("Content-Encoding", "gzip")
        if encoding == "br":
            ending = ".br"

        # and write the file
        BinaryWriter().write_file(base_name + ending, icap_body.res_body.body)
        
def main():

    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.DEBUG)
    
    # parse command line args
    parser = argparse.ArgumentParser(description='Parses out the contents of ICAP transactions.')
    parser.add_argument("--file", help="path to file to unpack, unpacked files be stored next to it.",required=True)
    
    args  = parser.parse_args()       
    unpack(args.file)

if __name__ == '__main__':
    main()
    
    