#!/usr/bin/env python3
import sys
import time
import json
import argparse
import requests

# global :(
g_verbose = False

def log(s):

    if g_verbose:
        sys.stderr.write("inspector_auth.py " + s)
        sys.stderr.flush()

#
#
#
def ip_to_user(server1, port1, server2, port2, token, src_ip):

    info = None

    # try first server
    try:

        url  = "http://%s:%s/ip/lookup/%s" % (server1, port1, src_ip)
        info = get_url(token, url)

    except Exception as e: 

        # rethrow if no second server configured
        if not server2:
            raise

        # try second server        
        url  = "http://%s:%s/ip/lookup/%s" % (server2, port2, src_ip)
        info = get_url(token, url)

    # we can only get here if info is not none
    assert(info is not None)

    # convert the JSON to our representation
    user = remap_user(info)

    # and return the user
    return user["user_name"]

#
# 
#
def process_input(args):

    # assign server info
    server1 = args.server1
    port1   = args.port1
    server2 = args.server2
    port2   = args.port2
    token   = args.token

    while 1:

        l = sys.stdin.readline().split()

        log(">>> %s\n" % l)

        try:
            src_ip = l[0]
            result = ip_to_user(server1, port1, server2, port2, token, src_ip)

            log('Success mapping %s to %s\n' % (src_ip, result))

            sys.stdout.write('OK user=%s\n' % result)

        except Exception as e:

            log('Failure mapping %s to user name; Error: %s\n' % (src_ip, str(e)))

            sys.stdout.write('ERR\n')

        sys.stdout.flush()

#
#
#
def remap_user(entry):

    result = {
        "user_ip"    : entry["IP"],
        "domain"     : "",
        "user_name"  : "",
        "user_pname" : "",
        "user_sid"   : ""
    }

    users = entry["users"]
    if users:

        user                 = users[0]        
        result["domain"]     = user["domain"]
        result["user_name"]  = user["name"]
        result["user_pname"] = user["principal"]
        result["user_sid"]   = user["sid"]

    return result

#
#
#
def remap_users(data):

    info = []
    for entry in data:
        info.append(remap_user(entry))

    return info

def get_url(token, url):

    log("get_url() - %s\n" % url)

    # ignore the proxy
    proxy   = ""
    proxies = {
        "http" : proxy,
        "https": proxy
    }

    # set the custom headers
    headers = {}
    if token:
        headers["Authorization"] = "Token %s" % token

    # issue the request
    req = requests.get(url, proxies=proxies, headers=headers)

    # and load the response
    info = json.loads(req.content)

    return info


#
#
#
def process_dump(args):

    # assign server info
    server1 = args.server1
    port1   = args.port1
    server2 = args.server2
    port2   = args.port2
    token   = args.token

    # let's go
    info = []
    e1   = None
    e2   = None

    # try server1 first
    try:

        url  = "http://%s:%s/ip/list/" % (server1, port1)
        info = get_url(token, url)

    except Exception as e: 
        e1 = e

    if e1 is None:

        print (json.dumps(remap_users(info)))
        return

    # well if we got here server1 failed, try server2 now
    try:

        url  = "http://%s:%s/ip/list/" % (server2, port2)
        info = get_url(token, url)

    except Exception as e:
        e2 = e

    if e2 is None:

        print (json.dumps(remap_users(info)))
        return

    # if we got here BOTH server1 and server2 failed, DUMP it
    raise Exception("DUMP failed\n\nServer1 Error:\n%s\n\nServer2 Error:\n%s\n\n" % (str(e1), str(e2)))

#
#
#
def process_args(args):

    if args.dump:
        process_dump(args)
    else:
        process_input(args)

#
# main
#
def main():

    # ok let's go
    start = time.time()
    
    # create parser
    parser = argparse.ArgumentParser(description='Implements pseudo authentication based in information collected by Active Directory Inspector (ADI) project https://github.com/diladele/active-directory-inspector. Usually used in transparent (intercept) deployments of Squid proxy.')

    # add args
    parser.add_argument("--server1", required=True, help="FQDN of the primary server with ADI installed")    
    parser.add_argument("--port1",   required=False, default="8000", help="port to connect to on primary server (default 8000)")
    parser.add_argument("--server2", required=False, help="FQDN of the backup server with ADI installed (optional)")
    parser.add_argument("--port2",   required=False, default="8000", help="port to connect to on backup server (default 8000)")
    parser.add_argument("--token",   required=False, help="authentication token to pass to ADI with every request")
    parser.add_argument("--dump",    required=False, action="store_true", help="dumps the mappings from configured servers as JSON")
    parser.add_argument("--verbose", required=False, action="store_true", help="increases the log level")
    
    # parse actual arguments
    args = parser.parse_args()

    if args.verbose:
        g_verbose = True

    # and do what is needed
    process_args(args)
    
if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)