#!/usr/bin/env python3
import os
import re
import sys
import time
import json
import urllib
import logging
import argparse
import requests
import traceback

#
#
#
class Config(object):

    def __init__(self, args):

        # read config file
        data = {}
        with open(args.conf, "r") as fin:
            data = json.load(fin)

        # set variables
        self.cache_clean  = data.get("cache_clean", 0)
        self.daemon_port  = data.get("daemon_port", 18890)
        self.deny_url     = data.get("deny_url", "http://blocked.example.lan")
        self.endpoint_url = "http://127.0.0.1:%d/check/request" % self.daemon_port

        # dump it
        logging.debug("Config info:")
        logging.debug("\tcache_clean => %s" % self.cache_clean)
        logging.debug("\tdeny_url    => %s" % self.deny_url)
        logging.debug("\tdaemon_port => %s" % self.daemon_port)


#
# we assume the following is in squid conf (default squid format with addition of some more information)
# where (see http://www.squid-cache.org/Doc/config/logformat), any non existent entry is a dash!
#
# url_rewrite_extras "%>a/%>A %un %>rm myip=%la myport=%lp bump_mode=%ssl::bump_mode sni=\"%ssl::>sni\" referer=\"%{Referer}>h\""
#
# >a  - client source IP address
# >A  - client FQDN (or IP address if no FQDN is available)
# un  - user name 
# >rm - request METHOD from client (GET/POST/CONNECT)
# 
class Request(object):

    def __init__(self, line):

        # debug check
        assert(len(line) > 0)

        # save for debug
        self.line = line

        # for now we do not support channel id, so first must be URL
        line = self.__parse_url(line)
        line = self.__parse_client(line)
        line = self.__parse_user(line)
        line = self.__parse_method(line)
        line = self.__parse_attributes(line)

        # debug check we parsed fully
        assert(len(line) == 0)

        # note for Google Safe Browsing URL *must* start with http:// or https://
        self.url = self.adjust_url(self.url, self.method)

    def adjust_url(self, url, method):

        if method.upper() == "CONNECT":
            return "https://" + url
        return url

    def as_dict(self):

        data = {
            "url"         : self.url,
            "client_ip"   : self.client_ip,
            "client_fqdn" : self.client_fqdn,
            "line"        : self.line,
            "username"    : self.username,
            "method"      : self.method
        }
        for key in self.attrs:
            if key not in data:
                data[key] = self.attrs[key]

        return data

    def __parse_url(self, line):

        pos = line.find(" ")
        if pos == -1:

            # technically it is possible to have just URL without any extra, so assign
            self.url = line

            # and return the rest
            return ""

        else:

            # assign
            self.url = line[:pos]

            # and return the rest
            return line[pos + 1:]

    def __parse_client(self, line):

        self.client_ip   = ""
        self.client_fqdn = ""

        if not line:
            return ""

        pos = line.find(" ")
        if pos == -1:
            
            # client ip/fqdn is all we have
            self.client_ip   = self.__parse_client_ip(line);
            self.client_fqdn = self.__parse_client_fqdn(line);

            # and return the rest
            return ""
            
        else:

            # assign and parse
            tmp = line[:pos]

            # client ip/fqdn is all we have
            self.client_ip   = self.__parse_client_ip(tmp);
            self.client_fqdn = self.__parse_client_fqdn(tmp);

            # and return the rest
            return line[pos + 1:]

    def __parse_user(self, line):

        self.username = ""

        if not line:
            return ""

        pos = line.find(" ")
        if pos == -1:
            
            # assign user name
            self.username = line;

            # and return the rest
            return ""
            
        else:

            # assign
            self.username = line[:pos]

            # and return the rest
            return line[pos + 1:]

    def __parse_method(self, line):

        self.method = ""

        if not line:
            return ""

        pos = line.find(" ")
        if pos == -1:
            
            # assign user name
            self.method = line;

            # and return the rest
            return ""
            
        else:

            # assign
            self.method = line[:pos]

            # and return the rest
            return line[pos + 1:]

    def __parse_client_ip(self, line):

        pos = line.find("/")
        if pos == -1:
            raise Exception("cannot find delimiter")

        return line[:pos]

    def __parse_client_fqdn(self, line):

        pos = line.find("/")
        if pos == -1:
            raise Exception("cannot find delimiter")

        return line[pos+1:]


    def __parse_attributes(self, line):

        # reset attributes
        self.attrs = {}

        while 1:

            pos = line.find("=")
            if pos == -1:
                break

            # get name
            name = line[:pos]
            line = line[pos + 1:].strip()

            # and value
            value = ""

            if True:

                # see what is the first character
                if not line.startswith("\""):

                    # it is a normal value and NOT a quoted string - look for the space or end
                    pos = line.find(" ");
                    if pos == -1:
                
                        # the whole string is a value
                        value = line

                        # do not forget to clear
                        line = ""

                    else:

                        # assign
                        value = line[:pos]

                        # clear
                        line = line[pos+1:]

                else:

                    # it is a quoted string, look for the closing quote
                    pos = line.find("\"", 1);
                    if -1 == pos:

                        # debug check
                        assert(len(line) > 0)

                        # ending quote not found, let all be the value
                        value = line[1:]

                        # clear
                        line = ""

                    else:

                        # assign
                        value = line[1: pos]

                        # clear and trim
                        line = line[pos+1:]
                        line = line.strip()

            # nice both name and value are parsed, assign and continue
            if name and value:
                self.attrs[name] = value

        # attributes are last so always return empty string
        return ""
#
#
#
class Response(object):
    pass

class InfectedResponse(Response):

    def __init__(self, request, data, deny_url):

        # debug check the data (data is actually the result of the scan)
        assert("safe" in data and data["safe"] == False)

        # collection of threats
        self.threats = []

        # enum threats
        for threat in data["threats"]:

            # and copy it
            self.threats.append(threat)

        # debug check we have at least one
        assert(len(self.threats) > 0)        

        # now calculate the reason why it was blocked and parameterize deny URL
        self.reasons = self.__calculate_reasons(self.threats)
        
        # construct deny URL
        if True:

            # append some params to deny URL
            args = { 
                'url'    : request.url, 
                'reason' : ",".join(list(set(self.reasons)))
            }

            # and save it
            self.deny_url = deny_url + "?" + urllib.parse.urlencode(args)

    def as_string(self):

        # convert reasons to one big string for logging
        reason = ",".join(list(set(self.reasons)))

        # and tell squid
        return "OK status=302 url=\"%s\" reason=\"%s\"" % (self.deny_url, reason)

    def __calculate_reasons(self, threats):

        values = []

        # loop over all threads
        for threat in self.threats:

            pattern           = threat.get("pattern", "unknown")
            threat_type       = threat.get("threat_type", 0)
            threat_entry_type = threat.get("threat_entry_type", 0)
            platform_type     = threat.get("platform_type", 0)

            # convert the thread type from int to string
            if threat_type == 1:
                threat_type = "MALWARE"
            elif threat_type == 2:
                threat_type = "SOCIAL_ENGINEERING"
            elif threat_type == 3:
                threat_type = "UNWANTED_SOFTWARE"
            elif threat_type == 4:
                threat_type = "POTENTIALLY_HARMFUL_APPLICATION"
            else:
                threat_type = "UNKNOWN"

            # convert platform type from int to string
            if platform_type == 1:
                platform_type = "WINDOWS"
            elif platform_type == 2:
                platform_type = "LINUX"
            elif platform_type == 3:
                platform_type = "ANDROID"
            elif platform_type == 4:
                platform_type = "OSX"
            elif platform_type == 5:
                platform_type = "IOS"
            elif platform_type == 6:
                platform_type = "ANY_PLATFORM"
            elif platform_type == 7:
                platform_type = "ALL_PLATFORMS"
            elif platform_type == 8:
                platform_type = "CHROME"
            else:
                platform_type = "UNKNOWN"

            # convert thread entry type from int into string
            if threat_entry_type == 1:
                threat_entry_type = "URL"
            elif threat_entry_type == 2:
                threat_entry_type = "EXECUTABLE"
            elif threat_entry_type == 3:
                threat_entry_type = "IP_RANGE"
            else:
                threat_entry_type = "UNKNOWN"

            # make the id string
            thread_id = "%s/%s/%s:%s" % (threat_type, platform_type, threat_entry_type, pattern)

            # and add it
            values.append(thread_id)

        return values

class CleanResponse(Response):

    def as_string(self):

        return "ERR"

#
#
#
class Envelope(object):

    def __init__(self, config):

        self.config = config

    def wrap(self, request):

        data = {
            "request" : request.as_dict()
        }
        return json.dumps(data)

#
#
#
class Cache(object):

    def __init__(self):

        # map of url to request, response tuple
        self.entries = {}

    def find(self, request):

        # see if the url is in cache
        if request.url not in self.entries:
            return None

        # ok, get the entry
        entry = self.entries[request.url]

        # shall invalidate
        if self.__stale(entry):

            # entry stale, remove
            self.entries.pop(request.url, None)

            # and return nothing
            return None

        # good, entry is still valid, unpack it
        (expiry_time, dummy, response) = entry

        # and return
        return response

    def __stale(self, entry):

        # unpack it
        (expiry_time, request, response) = entry

        # see if the time to expire has arrived
        if time.time() > expiry_time:
            return True

        return False

    def append(self, expiry_time, request, response):

        # make a tuple
        entry = (expiry_time, request, response)

        # and push it
        self.entries[request.url] = entry

#
#
#
class Processor(object):

    def __init__(self, config):

        self.cache  = Cache()
        self.config = config

    def process(self, request):

        # there are URL schemes which we do not support
        if request.url.startswith("cache_object://"):
            return CleanResponse()

        # see if url is in cache
        cached = self.cache.find(request)
        if cached is not None:

            # debug check
            assert(isinstance(cached, InfectedResponse) or isinstance(cached, CleanResponse))

            # and return it
            return cached

        # pack the URL
        data = Envelope(self.config).wrap(request)

        # ignore the proxy
        proxies = {
            "http" : None,
            "https": None,
        }

        # post the request
        r = requests.post(self.config.endpoint_url, data, proxies=proxies)
        if r.status_code != 200:

            # server failed, raise and error
            raw_text = r.text
            raw_text = raw_text.replace("\n", "_")
            raw_text = raw_text.replace("\r", "_")

            raise Exception("POST failed; status %d, response (as one line - %s)" % (r.status_code, raw_text))

        # get the output
        output = r.json()

        # and dump it
        logging.debug("wsgsbd daemon replied: %s" % r.json())

        # see if there were any matches
        if "safe" in output and output["safe"] == False:

            # ok url is infected, create the response
            response = InfectedResponse(request, output, self.config.deny_url)

            # create expiry time (note the infected responses are cached correctly in wsgsbd daemon, here we just doing blank cache as it is better to overblock than to underblock!)
            expiry_time = time.time() + self.config.cache_clean

            # save it to cache
            self.cache.append(expiry_time, request, response)

            # and return it
            return response

        # good, no matches, create clean response
        response = CleanResponse()

        # see if we have to cache clean responses
        if self.config.cache_clean != 0:

            # create expiry time
            expiry_time = time.time() + self.config.cache_clean

            # save it to cache too
            self.cache.append(expiry_time, request, response)

        # and return nicely
        return response


#
#
#
def run(config):

    # create the processor
    processor = Processor(config)

    # loop until squid closes the stdin
    while 1:

        # get line
        line = sys.stdin.readline()
        if len(line) == 0:
            break

        # dump it
        logging.debug(">>> %s\n" % line)

        try:

            # create request
            r = Request(line)

            # process it
            response = processor.process(r)

            # dump for squid
            sys.stdout.write("%s\n" % response.as_string())

            # and for log just in case
            logging.debug("<<< %s\n" % response.as_string())

        except Exception as e:

            # dump error
            error = str(e) + traceback.format_exc()
            error = error.replace("\r", "")
            error = error.replace("\n", "_")

            # tell squid something is wrong
            sys.stdout.write("BH %s\n" % error)

            # and log it
            logging.error("XXX %s\n" % error)


        # explicitly flush the stdout
        sys.stdout.flush()


#
#
#
def main():

    # define arguments
    parser = argparse.ArgumentParser(description='Safe Browsing Guard URL Rewriter for Squid Proxy, (c) Diladele B.V., 2017.')
    parser.add_argument("--conf", help="Path to config file, for example /opt/websafety/etc/antivirus/safe_browsing.json", required=True)
    parser.add_argument("--verbose", help="output verbose log", action="store_true")

    # parse them
    args = parser.parse_args()    

    # make log write to stderr
    logging.basicConfig(format='%(asctime)s SBG-%(process)d %(levelname)s %(message)s', level=[logging.INFO, logging.DEBUG][args.verbose], stream=sys.stderr)   

    # start
    logging.info("%s is starting." % os.path.basename(__file__))

    # create configuration
    config = Config(args)
        
    # and run until stopped by squid
    run(config)

    # tell it
    logging.info("%s is stopped." % os.path.basename(__file__))



#
#
#
class DebugConfig(object):

    def __init__(self):

        self.cache_clean  = 60
        self.daemon_port  = 18890
        self.deny_url     = "https://www.diladele.com"
        self.endpoint_url = "http://127.0.0.1:%d/check/request" % self.daemon_port

def test():

    # create debug config
    config = DebugConfig()

    # create the processor
    processor = Processor(config)

    # process some test lines
    lines = [
        'http://malware.wicar.org/data/eicar.com',
        'http://malware.wicar.org/data/eicar.com',
        'http://www.test.com/test.html 192.168.1.112/192.168.1.112 - GET myip=192.168.1.112 myport=3128 bump_mode=- sni="-" referer="http://www.test.com/index.html"',
        'http://www.test.com/test.html 192.168.1.112/192.168.1.112 - GET myip=192.168.1.112 myport=3128 bump_mode=- sni="-" referer="http://www.test.com/index.html"', # to test cache

    ]

    for line in lines:

        request  = Request(line)
        response = processor.process(request)

        print (response.as_string())

#
#
#
if __name__ == '__main__':

    # test()
    main()
    