import json
import traceback

#
# django
#
from django import forms
from django.views import generic
from django.http import HttpResponse
from django.core.paginator import Paginator

#
# domain
#
from _domain.traffic import *

#
#
#
class RealTimeRawJson(generic.View):

    def get(self, request, *args, **kwargs):

        # allocate default response
        data = { 'error_text' : '', 'traceback' : '', 'values' : [] }

        try:

            data['values'] = RealTimeJsonProvider().run({})

        except Exception as e:

            data['error_text'] = str(e)
            data['traceback']  = str(traceback.format_exc())

        return HttpResponse(json.dumps([data], ensure_ascii=True), content_type='application/json')
#
#
#
class RealTimeSummary(generic.TemplateView):

    template_name = 'traffic/realtime/summary.html'

    def get_context_data(self, **kwargs):

        context  = super(RealTimeSummary, self).get_context_data(**kwargs)

        try:
            # create the frame
            frame = FrameProvider.instance()

            # crete views
            mgr     = SummaryMgr(frame)
            users   = UsersSummary(frame)
            domains = DomainsSummary(frame)

            # and fill the context
            context['summary']               = mgr.summary()
            context['modules']               = mgr.modules()
            context['categories']            = mgr.categories()            
            context['top_n_domains']         = domains.top_n_browsing(15)
            context['top_n_blocked_domains'] = domains.top_n_blocked(15)            
            context['top_n_users']           = users.top_n_browsing(10)
            context['top_n_blocked_users']   = users.top_n_blocked(10)
            context['hits_over_time']        = mgr.hits_over_time()

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

#
#
#
class RealTimeTable(generic.TemplateView):

    template_name = 'traffic/realtime/table.html'

    def get_current_page(self):

        # calculate what page to show
        page_n = 1

        try:
            value  = self.request.GET.get('page', 1)
            value  = int(value)            
            page_n = value
        except Exception as e:
            pass

        return page_n

    def get_context_data(self, **kwargs):

        context = super(RealTimeTable, self).get_context_data(**kwargs)

        try:
            # construct the options
            options = Options()
            if True:
                options.where             = self.request.GET.get("q", "")
                options.order.column_name = self.request.GET.get("o", "timestamp")                
                options.order.ascending   = [True, False][self.request.GET.get("a", "-") == "-"]
                
            # get the objects from wsmgrd
            object_list = TableMgr().retrieve(options)
                        
            # construct the paginator, current page and items
            paginator    = Paginator(object_list, 100)
            current_page = self.get_current_page()
            items        = paginator.page(current_page)

            # list of pages for quick pagination
            if True:

                min_value = 1
                max_value = 10

                total_pages = paginator.num_pages
                if total_pages < 10:
                    max_value = total_pages


                min_value = max(0, current_page - 5)
                max_value = min(current_page + 5, total_pages)

            # assign the context
            context['page']      = items
            context['paginator'] = paginator
            context['pages']     = [i for i in range(min_value + 1, max_value + 1)]
            context['q']         = options.where
            context['o']         = options.order.column_name
            context['a']         = ["+", "-"][options.order.ascending == False]

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

#
#
#
class RealTimeUserList(generic.TemplateView):

    template_name = 'traffic/realtime/user_list.html'

    def get_context_data(self, **kwargs):

        context = super(RealTimeUserList, self).get_context_data(**kwargs)

        try:
            summary = UsersSummary(FrameProvider.instance())

            context['top_n_browsing'] = summary.top_n_browsing()
            context['top_n_blocked']  = summary.top_n_blocked()
            
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

#
# list of all or blocked domains for a given user
#
class RealTimeUserDomainList(generic.TemplateView):

    template_name = 'traffic/realtime/user_domain_list.html'

    def get_context_data(self, **kwargs):

        context = super(RealTimeUserDomainList, self).get_context_data(**kwargs)

        try:
            # get the user
            u = self.kwargs['u']
            if not u:
                raise Exception("User name or IP address is not specified")

            # get the blocked
            blocked = False
            if self.request.GET.get("blocked", "0") == "1":
                blocked = True

            # get the records
            table   = TableUser(FrameProvider.instance(), u)
            records = table.retrieve(
                [HitsType.ALL_AVAILABLE, HitsType.ONLY_BLOCKED][blocked]
            )

            # and render
            context['blocked'] = blocked
            context['u']       = u
            context['records'] = records

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

#
# list of ULRs for a given user and given domain
#
class RealTimeUserDomainDetail(generic.TemplateView):

    template_name = 'traffic/realtime/user_domain_detail.html'

    def get_context_data(self, **kwargs):

        context = super(RealTimeUserDomainDetail, self).get_context_data(**kwargs)

        try:
            # get the user and domain
            u = self.kwargs['u']
            d = self.kwargs['d']
            if not d or not u:
                raise Exception("Domain name or user name or user IP is not specified")

            # get the blocked
            blocked = False
            if self.request.GET.get("blocked", "0") == "1":
                blocked = True

            table   = TableUserDomain(FrameProvider.instance(), u, d)
            records = table.retrieve(
                [HitsType.ALL_AVAILABLE, HitsType.ONLY_BLOCKED][blocked]
            )

            context['blocked'] = blocked
            context['u']       = u 
            context['d']       = d 
            context['records'] = records

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context


#
#
#
class RealTimeDomainList(generic.TemplateView):

    template_name = 'traffic/realtime/domain_list.html'

    def get_context_data(self, **kwargs):

        context = super(RealTimeDomainList, self).get_context_data(**kwargs)

        try:

            summary = DomainsSummary(FrameProvider.instance())

            context['top_n_browsing'] = summary.top_n_browsing()
            context['top_n_blocked']  = summary.top_n_blocked()
            
        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

class RealTimeDomainUserList(generic.TemplateView):

    template_name = 'traffic/realtime/domain_user_list.html'

    def get_context_data(self, **kwargs):

        context = super(RealTimeDomainUserList, self).get_context_data(**kwargs)

        try:
            # get the domain
            d = self.kwargs['d']
            if not d:
                raise Exception("Domain name is not specified")

            # get the blocked
            blocked = False
            if self.request.GET.get("blocked", "0") == "1":
                blocked = True

            # get the records
            table   = TableDomain(FrameProvider.instance(), d)
            records = table.retrieve(
                [HitsType.ALL_AVAILABLE, HitsType.ONLY_BLOCKED][blocked]
            )

            # and render
            context['blocked'] = blocked
            context['d']       = d
            context['records'] = records

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

class RealTimeDomainUserDetail(generic.TemplateView):

    template_name = 'traffic/realtime/domain_user_detail.html'

    def get_context_data(self, **kwargs):

        context = super(RealTimeDomainUserDetail, self).get_context_data(**kwargs)

        try:
            # get the user and domain
            u = self.kwargs['u']
            d = self.kwargs['d']
            if not d or not u:
                raise Exception("Domain name or user name or user IP is not specified")

            # get the blocked
            blocked = False
            if self.request.GET.get("blocked", "0") == "1":
                blocked = True

            table   = TableDomainUser(FrameProvider.instance(), d, u)
            records = table.retrieve(
                [HitsType.ALL_AVAILABLE, HitsType.ONLY_BLOCKED][blocked]
            )

            context['blocked'] = blocked
            context['u']       = u 
            context['d']       = d 
            context['records'] = records

        except Exception as e:

            context['error_text'] = str(e)
            context['traceback']  = str(traceback.format_exc())

        return context

'''
class View_MonitorSurfingNow(FormListView):
    lastAsc = False

    def get(self, request, *args, **kwargs):

        order_by = request.GET.get('order_by')
        previous_order_by = request.GET.get('pob')
        name = self.request.GET.get('cn')
        value = self.request.GET.get('cv')
        asc = self.request.GET.get('asc', False)
        header_click = self.request.GET.get('reverse_asc', False)
        submitted_thru_form = request.GET.get('column_name', False)

        form = SurfingNowForm(self.request.GET)

        if name and name.isdigit():
            column_name = int(name)
            column_value = value
        elif form.is_valid():
            column_name = int(form.cleaned_data['column_name'])
            column_value = form.cleaned_data['column_value']
            order_by = "timestamp"
        else:
            column_name = SurfingNow.COLUMN_NAME_NONE
            column_value = ""
            order_by = "timestamp"

        if submitted_thru_form:
            asc = False
            order_by = "timestamp"

        if header_click:
            if order_by == previous_order_by and order_by is not None and order_by != '':
                if asc == 'True':
                    asc = False
                else:
                    asc = True
            else:
                if order_by == 'timestamp' or order_by == 'iid':
                    asc = False
                else:
                    asc = True

        previous_order_by = order_by

        object_list = HistoryManager().get_all(SurfingNow.COLUMN_NAME_TO_DB_COLUMN[column_name], column_value, order_by, 1000, asc)
        paginator = Paginator(object_list, 100)
        page = request.GET.get('page')

        try:
            items = paginator.page(page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            items = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            items = paginator.page(paginator.num_pages)

        ctx = {
            'form':form,
            'items':items,
            'paginator' : paginator,
            'page_obj' : items,
            'cn': column_name,
            'cv': column_value,
            'order_by': order_by,
            'page': page,
            'ascending': asc,
            'pob': previous_order_by }

        return render(request, 'traffic/monitor/surfing_all.html', ctx)
'''